This is the mail archive of the gdb-patches@sourceware.org mailing list for the GDB project.


Index Nav: [Date Index] [Subject Index] [Author Index] [Thread Index]
Message Nav: [Date Prev] [Date Next] [Thread Prev] [Thread Next]
Other format: [Raw text]

Re: [PATCH,gdb, Update1]: ensures that cie ptr of an fda is a cie


On Jul 4, 2011, at 8:41 PM, ext Joel Brobecker wrote:

>> This increases the robustness of gdb, so I think it is worthwhile to add.
> 
> Indeed.
> [...]

Thanks for the comments Joel, here is a revised patch that hopefully addresses your issues

Fawzi

2011-07-04 Fawzi Mohamed <fawzi.mohamed@nokia.com>

	* dwarf2-frame.c (decode_frame_entry, decode_frame_entry_1): ensure
	that CIE pointer of an FDE really points to a CIE 

Index: gdb/dwarf2-frame.c
===================================================================
RCS file: /cvs/src/src/gdb/dwarf2-frame.c,v
retrieving revision 1.125
diff -p -d -u -r1.125 dwarf2-frame.c
--- gdb/dwarf2-frame.c	4 Jul 2011 16:30:09 -0000	1.125
+++ gdb/dwarf2-frame.c	5 Jul 2011 09:07:59 -0000
@@ -1801,17 +1801,28 @@ add_fde (struct dwarf2_fde_table *fde_ta
 #define DW64_CIE_ID ~0
 #endif
 
+/* defines the type of eh_frames that are expected to be decoded: CIE, FDE
+   or any of them */
+enum eh_frame_type
+{
+  eh_cie_type_id = 1<<0,
+  eh_fde_type_id = 1<<1,
+  eh_cie_or_fde_type_id = eh_cie_type_id + eh_fde_type_id
+};
+
 static gdb_byte *decode_frame_entry (struct comp_unit *unit, gdb_byte *start,
 				     int eh_frame_p,
                                      struct dwarf2_cie_table *cie_table,
-                                     struct dwarf2_fde_table *fde_table);
+                                     struct dwarf2_fde_table *fde_table,
+                                     enum eh_frame_type entry_type);
 
-/* Decode the next CIE or FDE.  Return NULL if invalid input, otherwise
-   the next byte to be processed.  */
+/* Decode the next CIE or FDE, entry type specifies the expected type.
+   Return NULL if invalid input, otherwise the next byte to be processed.  */
 static gdb_byte *
 decode_frame_entry_1 (struct comp_unit *unit, gdb_byte *start, int eh_frame_p,
                       struct dwarf2_cie_table *cie_table,
-                      struct dwarf2_fde_table *fde_table)
+                      struct dwarf2_fde_table *fde_table,
+                      enum eh_frame_type entry_type)
 {
   struct gdbarch *gdbarch = get_objfile_arch (unit->objfile);
   gdb_byte *buf, *end;
@@ -1862,6 +1873,9 @@ decode_frame_entry_1 (struct comp_unit *
       char *augmentation;
       unsigned int cie_version;
 
+      /* checks that we expected a CIE */
+      gdb_assert ((entry_type & eh_cie_type_id) != 0);
+
       /* Record the offset into the .debug_frame section of this CIE.  */
       cie_pointer = start - unit->dwarf_frame_buffer;
 
@@ -2027,6 +2041,9 @@ decode_frame_entry_1 (struct comp_unit *
       /* This is a FDE.  */
       struct dwarf2_fde *fde;
 
+      /* checks that we expected a FDE */
+      gdb_assert ((entry_type & eh_fde_type_id) != 0);
+
       /* In an .eh_frame section, the CIE pointer is the delta between the
 	 address within the FDE where the CIE pointer is stored and the
 	 address of the CIE.  Convert it to an offset into the .eh_frame
@@ -2048,7 +2065,8 @@ decode_frame_entry_1 (struct comp_unit *
       if (fde->cie == NULL)
 	{
 	  decode_frame_entry (unit, unit->dwarf_frame_buffer + cie_pointer,
-			      eh_frame_p, cie_table, fde_table);
+			      eh_frame_p, cie_table, fde_table,
+			      eh_cie_type_id);
 	  fde->cie = find_cie (cie_table, cie_pointer);
 	}
 
@@ -2089,11 +2107,12 @@ decode_frame_entry_1 (struct comp_unit *
   return end;
 }
 
 /* Read a CIE or FDE in BUF and decode it.  */
 static gdb_byte *
 decode_frame_entry (struct comp_unit *unit, gdb_byte *start, int eh_frame_p,
                     struct dwarf2_cie_table *cie_table,
-                    struct dwarf2_fde_table *fde_table)
+                    struct dwarf2_fde_table *fde_table,
+                    enum eh_frame_type entry_type)
 {
   enum { NONE, ALIGN4, ALIGN8, FAIL } workaround = NONE;
   gdb_byte *ret;
@@ -2102,7 +2121,8 @@ decode_frame_entry (struct comp_unit *un
   while (1)
     {
       ret = decode_frame_entry_1 (unit, start, eh_frame_p,
-                                  cie_table, fde_table);
+                                  cie_table, fde_table,entry_type,
+                                  entry_type);
       if (ret != NULL)
 	break;
 
@@ -2256,7 +2276,8 @@ dwarf2_build_frame_info (struct objfile 
           frame_ptr = unit->dwarf_frame_buffer;
           while (frame_ptr < unit->dwarf_frame_buffer + unit->dwarf_frame_size)
             frame_ptr = decode_frame_entry (unit, frame_ptr, 1,
-                                            &cie_table, &fde_table);
+                                            &cie_table, &fde_table,
+                                            eh_cie_or_fde_type_id);
 
           if (cie_table.num_entries != 0)
             {
@@ -2089,11 +2107,12 @@ decode_frame_entry_1 (struct comp_unit *
 }
 
-/* Read a CIE or FDE in BUF and decode it.  */
+/* Read a CIE or FDE in BUF and decode it. entry type specifies if we expect an FDE or a CIE */
 static gdb_byte *
 decode_frame_entry (struct comp_unit *unit, gdb_byte *start, int eh_frame_p,


Index Nav: [Date Index] [Subject Index] [Author Index] [Thread Index]
Message Nav: [Date Prev] [Date Next] [Thread Prev] [Thread Next]